/*
 * (c) Copyright IBM Corp. 2000, 2002.
 * All Rights Reserved.
 */
package org.eclipse.tomcat.internal;

import java.io.*;
import java.io.PrintStream;
import java.net.*;

import org.apache.catalina.*;
import org.apache.catalina.core.StandardContext;
import org.apache.catalina.startup.Embedded;
import org.eclipse.core.runtime.Platform;
import org.eclipse.help.IAppServer;

public class TomcatAppServer implements IAppServer {

	private Embedded tomcat;
	private Engine engine;
	private Host host;
	String hostAddress;
	int port;
	private boolean isStarted = false;

	public TomcatAppServer() {
		port = TomcatPlugin.getDefault().getPluginPreferences().getInt(TomcatPlugin.PORT_KEY);
		
		hostAddress = TomcatPlugin.getDefault().getPluginPreferences().getString(TomcatPlugin.HOST_KEY);
		if (hostAddress != null && hostAddress.trim().length() == 0)
			hostAddress = null;
		
		tomcat = TomcatPlugin.getDefault().getTomcat();
	}

	/**
	 * Adds the specified webapp from a plugin.
	 * <p> It is assumed that webapp names are unique. It is suggested to
	 * create unique web app names by prefixing them with the plugin id.</p>
	 * @param webApp the name of the web application (context name)
	 * @param plugin the plugin identifier of the webapp
	 * @param path the plugin relative path where the web app WAR or directory is located
	 * @return true if the webapp was added, false otherwise
	 */
	public boolean add(String webApp, String plugin, String path) {
		try {
			//System.out.println("starting " + webApp);

			if (!isStarted)
				start();

			String realPath = getPluginLocation(plugin);
			if (path != null && path.length() > 0)
				realPath = realPath + path;
			else
				realPath = realPath;

			Context context = tomcat.createContext("/" + webApp, realPath);
			if (context instanceof StandardContext)
				 ((StandardContext) context).setWorkDir(getWorkingDirectory(webApp, plugin));

			ClassLoader loader = new TomcatClassLoader(plugin);
			context.setLoader(tomcat.createLoader(loader));

			host.addChild(context);
			return true;

		} catch (Exception exc) {
			tomcat.getLogger().log(exc, "[" + webApp + "," + plugin + "," + path + "]");
			return false;
		}
	}

	/**
	 * Stops the specified webapp
	 * @param webApp the name of the web app
	 * @param plugin the plugin containg the web app. Not used now.
	 */
	public void remove(String webApp, String plugin) {
		//System.out.println("stopping " + webApp);
		Context context = (Context) host.findChild("/" + webApp);
		tomcat.removeContext(context);
	}

	/**
	 * Returns the host name or ip the app server runs on.
	 */
	public String getHost() {
		return hostAddress;
	}

	/**
	 * Returns the port number the app server listens on.
	 */
	public int getPort() {
		return port;
	}

	/**
	 * Sets the host and port for the app server. Must be called before start().
	 * @param host the host name or IP address. Pass null when any address on 
	 *         local machine is to be used.
	 * @param port the port number to be used. Pass 0 to let the system select
	 *         an available port.
	 */
	public void setAddress(String hostAddress, int port) {
		this.hostAddress = hostAddress;
		this.port = port;

	}

	/**
	 * Starts the app server
	 */
	public boolean start() {
		if (isStarted)
			return true;

		try {

			String home = System.getProperty("catalina.home");

			// Create a very basic container hierarchy
			engine = tomcat.createEngine();
			engine.setDefaultHost("localhost");

			host = tomcat.createHost("localhost", home + "/webapps");
			engine.addChild(host);

			// Install the assembled container hierarchy
			PrintStream sysOut = System.out;
			// reassign standard output to prevent Tomcat from writing
			// its version message there.
			System.setOut(new PrintStream(new ByteArrayOutputStream()));
			try {
				tomcat.addEngine(engine);
			} catch (Exception exception) {
				System.setOut(sysOut);
				throw exception;
			}
			System.setOut(sysOut);

			// Root context
			Context root = tomcat.createContext("", home + "/webapps/ROOT");
			// this line should be replaced once tomcat provides support
			// for setting the working directory
			if (root instanceof StandardContext)
				((StandardContext) root).setWorkDir(
					getWorkingDirectory("ROOT", "org.eclipse.tomcat"));
			root.setLoader(tomcat.createLoader(this.getClass().getClassLoader()));
			host.addChild(root);

			// Create Connector
			InetAddress iAddress = null;
			if (hostAddress != null) {
				try {
					iAddress = InetAddress.getByName(hostAddress);
				} catch (UnknownHostException uhe) {
					// will default to all interfaces
				}
			}
			Connector connector = tomcat.createConnector(iAddress, port, false);
			PortTrackingServerSocketFactory sSocketFactory =
				new PortTrackingServerSocketFactory();
			connector.setFactory(sSocketFactory);

			// add Connector to Tomcat
			tomcat.addConnector(connector);

			// if port dynamicaly assigned, obtain port number
			// from the factory
			if (port == 0) {
				port = sSocketFactory.getAssignedPort();

			}
			// if null passed for hostAddress, determine Address
			// that is available for local connections
			if (hostAddress == null) {
				hostAddress = LocalConnectionTest.getLocalInterface(port);
			}

			isStarted = true;
			return true;
		} catch (Exception exc) {
			System.out.println(exc.getMessage());
			return false;
		}
	}

	/**
	 * Stops the app server
	 */
	public boolean stop() {
		if (!isStarted)
			return true;

		try {
			// Remove the engine (which should trigger removing the connector)
			tomcat.removeEngine(engine);
		} catch (Exception exc) {
		}
		return true;
	}

	private String getPluginLocation(String pluginId) {
		try {
			URL installURL =
				Platform.getPluginRegistry().getPluginDescriptor(pluginId).getInstallURL();

			return Platform.resolve(installURL).getFile();
		} catch (Exception exc) {
			System.out.println("cannot find plugin " + pluginId);
			return null;
		}
	}

	private String getWorkingDirectory(String webApp, String plugin) {
		return TomcatPlugin
			.getDefault()
			.getStateLocation()
			.append(plugin)
			.append(webApp)
			.toOSString();
	}

}